package es.uv.garcosda.mongo.controllers;

import es.uv.garcosda.mongo.domain.User;
import es.uv.garcosda.mongo.services.UserService;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/api/v1/users")
public class UserController {

	@Autowired
	UserService us;

	@GetMapping
	public ResponseEntity<List<User>> getAll(@RequestParam(value="firstname", required = false) Optional<String> first, 
			@RequestParam(value="lastname", required = false) Optional<String> last, 
			@RequestParam(value="start", required = false) Optional<Boolean> start) {
		List<User> users = new ArrayList<User>();
		if(first.isPresent()) {
			if(start.isPresent()) {
				users = this.us.findByFirstnameStartingWith(first.get());
			}
			else {
				users = this.us.findByFirstname(first.get());
			}
		}
		else if(last.isPresent()) {
			if(start.isPresent()) {
				users = this.us.findByLastnameStartingWith(last.get());
			}
			else {
				users = this.us.findByLastname(last.get());
			}
		}
		else {
			users = us.findAll();
		}
		
		return new ResponseEntity<>(users, HttpStatus.OK);
	}

	@GetMapping("/{id}")
	public ResponseEntity<?> getById(@PathVariable Integer id) {
		User u = us.findById(id);
		if (u == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}

		return new ResponseEntity<>(u, HttpStatus.OK);
	}

	@PostMapping
	public ResponseEntity<?> create(@RequestBody User user) throws IOException {
		User u = us.create(user);
		if (u == null) {
			return new ResponseEntity<>(HttpStatus.BAD_REQUEST);
		}

		return new ResponseEntity<>(u, HttpStatus.CREATED);
	}

	@DeleteMapping("/{id}")
	public ResponseEntity<?> delete(@PathVariable Integer id) {
		User u = us.findById(id);
		if (u == null) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		us.delete(u);
		return new ResponseEntity<>(u, HttpStatus.OK);
	}
}
